@@ -4,6 +4,7 @@ import sys, getopt, time, os
 import pexpect, re
 import telnetlib
 import atexit
+import __main__
 
 ## do not add code here.
 #BEGIN_VERSION_GENERATION
@@ -62,10 +63,22 @@ all_opt = {
 		"required" : "0",
 		"shortdesc" : "Write debug information to given file",
 		"order" : 52 },
+	"delay" : {
+		"getopt" : "f:",
+		"longopt" : "delay",
+		"help" : "--delay <seconds>              Wait X seconds before fencing is started",
+		"required" : "0",
+		"shortdesc" : "Wait X seconds before fencing is started",
+		"default" : "0",
+		"order" : 200 },
 	"agent"   : {
 		"getopt" : "",
 		"help" : "",
 		"order" : 1 },
+	"web"    : {
+		"getopt" : "",
+		"help" : "",
+		"order" : 1 },
 	"action" : {
 		"getopt" : "o:",
 		"longopt" : "action",
@@ -202,6 +215,14 @@ all_opt = {
 		"required" : "0",
 		"shortdesc" : "Physical switch number on device",
 		"order" : 1 },
+	"suborg" : {
+		"getopt" : "s:",
+		"longopt" : "suborg",
+		"help" : "--suborg=<path>                Additional path needed to access suborganization",
+		"required" : "0",
+		"shortdesc" : "Additional path needed to access suborganization",
+		"default" : "",
+		"order" : 1 },
 	"partition" : {
 		"getopt" : "n:",
 		"help" : "-n <id>                        Name of the partition",
@@ -324,32 +345,66 @@ all_opt = {
 		"longopt" : "login-timeout",
 		"help" : "--login-timeout <seconds>      Wait X seconds for cmd prompt after login",
 		"default" : "5", 
+		"required" : "0",
+		"shortdesc" : "Wait X seconds for cmd prompt after login",
 		"order" : 200 },
 	"shell_timeout" : {
 		"getopt" : "Y:",
 		"longopt" : "shell-timeout",
 		"help" : "--shell-timeout <seconds>      Wait X seconds for cmd prompt after issuing command",
 		"default" : "3", 
+		"required" : "0",
+		"shortdesc" : "Wait X seconds for cmd prompt after issuing command",
 		"order" : 200 },
 	"power_timeout" : {
 		"getopt" : "g:",
 		"longopt" : "power-timeout",
 		"help" : "--power-timeout <seconds>      Test X seconds for status change after ON/OFF",
 		"default" : "20", 
+		"required" : "0",
+		"shortdesc" : "Test X seconds for status change after ON/OFF",
 		"order" : 200 },
 	"power_wait" : {
 		"getopt" : "G:",
 		"longopt" : "power-wait",
 		"help" : "--power-wait <seconds>         Wait X seconds after issuing ON/OFF",
 		"default" : "0", 
+		"required" : "0",
+		"shortdesc" : "Wait X seconds after issuing ON/OFF",
 		"order" : 200 },
 	"missing_as_off" : {
 		"getopt" : "M",
 		"longopt" : "missing-as-off",
 		"help" : "--missing-as-off               Missing port returns OFF instead of failure",
-		"order" : 200}
+		"required" : "0",
+		"shortdesc" : "Missing port returns OFF instead of failure",
+		"order" : 200 },
+	"retry_on" : {
+		"getopt" : "F:",
+		"longopt" : "retry-on",
+		"help" : "--retry-on <attempts>          Count of attempts to retry power on",
+		"default" : "1",
+		"required" : "0",
+		"shortdesc" : "Count of attempts to retry power on",
+		"order" : 201 },
+	"session_url" : {
+		"getopt" : "s:",
+		"longopt" : "session-url",
+		"help" : "-s, --session-url              URL to connect to XenServer on.",
+		"required" : "1",
+		"shortdesc" : "The URL of the XenServer host.",
+		"order" : 1},
+	"uuid" : {
+		"getopt" : "U:",
+		"longopt" : "uuid",
+		"help" : "-U, --uuid                     UUID of the VM to fence.",
+		"required" : "0",
+		"shortdesc" : "The UUID of the virtual machine to fence.",
+		"order" : 1}
 }
 
+common_opt = [ "retry_on", "delay" ]
+
 class fspawn(pexpect.spawn):
 	def log_expect(self, options, pattern, timeout):
 		result = self.expect(pattern, timeout)
@@ -412,6 +467,8 @@ def metadata(avail_opt, options, docs):
 	print "<?xml version=\"1.0\" ?>"
 	print "<resource-agent name=\"" + os.path.basename(sys.argv[0]) + "\" shortdesc=\"" + docs["shortdesc"] + "\" >"
 	print "<longdesc>" + docs["longdesc"] + "</longdesc>"
+	if docs.has_key("vendorurl"):
+		print "<vendor-url>" + docs["vendorurl"] + "</vendor-url>"
 	print "<parameters>"
 	for option, value in sorted_list:
 		if all_opt[option].has_key("shortdesc"):
@@ -419,7 +476,7 @@ def metadata(avail_opt, options, docs):
 
 			default = ""
 			if all_opt[option].has_key("default"):
-				default = "default=\""+all_opt[option]["default"]+"\""
+				default = "default=\""+str(all_opt[option]["default"])+"\""
 			elif options.has_key("-" + all_opt[option]["getopt"][:-1]):
 				if options["-" + all_opt[option]["getopt"][:-1]]:
 					try:
@@ -427,7 +484,7 @@ def metadata(avail_opt, options, docs):
 					except TypeError:
 						## @todo/@note: Currently there is no clean way how to handle lists
 						## we can create a string from it but we can't set it on command line
-						default = "default=\"list of values\""
+						default = "default=\"" + str(options["-" + all_opt[option]["getopt"][:-1]]) +"\""
 			elif options.has_key("-" + all_opt[option]["getopt"]):
 				default = "default=\"true\" "
 
@@ -448,9 +505,14 @@ def metadata(avail_opt, options, docs):
 			print "\t</parameter>"
 	print "</parameters>"
 	print "<actions>"
-	print "\t<action name=\"on\" />"
-	print "\t<action name=\"off\" />"
-	print "\t<action name=\"reboot\" />"
+	if avail_opt.count("io_fencing") == 0:
+		print "\t<action name=\"on\" />"
+		print "\t<action name=\"off\" />"
+		print "\t<action name=\"reboot\" />"
+	else:
+		print "\t<action name=\"enable\" />"
+		print "\t<action name=\"disable\" />"	
+
 	print "\t<action name=\"status\" />"
 	print "\t<action name=\"list\" />"
 	print "\t<action name=\"monitor\" />"
@@ -460,6 +522,12 @@ def metadata(avail_opt, options, docs):
 
 def process_input(avail_opt):
 	global all_opt
+	global common_opt
+
+	##
+	## Add options which are available for every fence agent
+	#####
+	avail_opt.extend(common_opt)
 
 	##
 	## Set standard environment
@@ -484,6 +552,11 @@ def process_input(avail_opt):
 			else:
 				longopt_list.append(all_opt[k]["longopt"])
 
+	## Compatibility layer
+	if avail_opt.count("module_name") == 1:
+		getopt_string += "n:"
+		longopt_list.append("plug=")
+	
 	##
 	## Read options from command line or standard input
 	#####
@@ -510,6 +583,8 @@ def process_input(avail_opt):
 		z = dict(opt)
 		if z.has_key("-T") == 1:
 			z["-o"] = "status"
+		if z.has_key("-n") == 1:
+			z["-m"] = z["-n"]
 
 		opt = z
 		##
@@ -539,6 +614,8 @@ def process_input(avail_opt):
 				name = "module_name"
 			elif name == "action" and 1 == avail_opt.count("io_fencing"):
 				name = "io_fencing"
+			elif name == "port" and 1 == avail_opt.count("drac_version"):
+				name = "module_name"
 
 			##
 			######
@@ -548,7 +625,7 @@ def process_input(avail_opt):
 
 			if all_opt[name]["getopt"].endswith(":"):
 				opt["-"+all_opt[name]["getopt"].rstrip(":")] = value
-			elif ((value == "1") or (value.lower() == "yes")):
+			elif ((value == "1") or (value.lower() == "yes") or (value.lower() == "on") or (value.lower() == "true")):
 				opt["-"+all_opt[name]["getopt"]] = "1"
 	return opt
 
@@ -559,7 +636,13 @@ def process_input(avail_opt):
 ######
 def check_input(device_opt, opt):
 	global all_opt
+	global common_opt
 
+	##
+	## Add options which are available for every fence agent
+	#####
+	device_opt.extend([x for x in common_opt if device_opt.count(x) == 0])
+	
 	options = dict(opt)
 	options["device_opt"] = device_opt
 
@@ -600,7 +683,7 @@ def check_input(device_opt, opt):
 	if (0 == options.has_key("-l")) and device_opt.count("login") and (device_opt.count("no_login") == 0):
 		fail_usage("Failed: You have to set login name")
 
-	if 0 == options.has_key("-a"):
+	if 0 == options.has_key("-a") and 0 == options.has_key("-s"):
 		fail_usage("Failed: You have to enter fence address")
 
 	if (device_opt.count("no_password") == 0):
@@ -618,7 +701,7 @@ def check_input(device_opt, opt):
 		if 0 == os.path.isfile(options["-k"]):
 			fail_usage("Failed: Identity file " + options["-k"] + " does not exist")
 
-	if (0 == ["list", "monitor"].count(options["-o"].lower())) and (0 == options.has_key("-n")) and (device_opt.count("port")):
+	if (0 == ["list", "monitor"].count(options["-o"].lower())) and (0 == options.has_key("-n") and 0 == options.has_key("-U")) and (device_opt.count("port")):
 		fail_usage("Failed: You have to enter plug number")
 
 	if options.has_key("-S"):
@@ -641,6 +724,8 @@ def check_input(device_opt, opt):
 			options["-u"] = 22
 		elif options.has_key("-z"):
 			options["-u"] = 443
+		elif device_opt.count("web"):
+			options["-u"] = 80
 		else:
 			options["-u"] = 23
 
@@ -673,8 +758,8 @@ def show_docs(options, docs = None):
 		sys.exit(0)
 
 	if options.has_key("-V"):
-		print RELEASE_VERSION, BUILD_DATE
-		print REDHAT_COPYRIGHT
+		print __main__.RELEASE_VERSION, __main__.BUILD_DATE
+		print __main__.REDHAT_COPYRIGHT
 		sys.exit(0)
 
 def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None):
@@ -682,7 +767,7 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 	
 	## Process options that manipulate fencing device
 	#####
-	if (options["-o"] == "list") and (0 == options["device_opt"].count("port")) and (0 == options["device_opt"].count("partition")):
+	if (options["-o"] == "list") and (0 == options["device_opt"].count("port")) and (0 == options["device_opt"].count("partition") and 0 == options["device_opt"].count("uuid")):
 		print "N/A"
 		return
 	elif (options["-o"] == "list" and get_outlet_list == None):
@@ -700,6 +785,8 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 				print o + options["-C"] + alias	
 		return
 
+	if options["-o"] in ["off", "reboot"]:
+		time.sleep(int(options["-f"]))
 	status = get_power_fn(tn, options)
 
 	if status != "on" and status != "off":  
@@ -715,9 +802,15 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 		if status == "on":
 			print "Success: Already ON"
 		else:
-			set_power_fn(tn, options)
-			time.sleep(int(options["-G"]))
-			if wait_power_status(tn, options, get_power_fn):
+			power_on = False
+			for i in range(1,1 + int(options["-F"])):
+				set_power_fn(tn, options)
+				time.sleep(int(options["-G"]))
+				if wait_power_status(tn, options, get_power_fn):
+					power_on = True
+					break
+
+			if power_on:
 				print "Success: Powered ON"
 			else:
 				fail(EC_WAITING_ON)
@@ -739,10 +832,19 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 			if wait_power_status(tn, options, get_power_fn) == 0:
 				fail(EC_WAITING_OFF)
 		options["-o"] = "on"
-		set_power_fn(tn, options)
-		time.sleep(int(options["-G"]))
-		if wait_power_status(tn, options, get_power_fn) == 0:
+
+		power_on = False
+		for i in range(1,1 + int(options["-F"])):
+			set_power_fn(tn, options)
+			time.sleep(int(options["-G"]))
+			if wait_power_status(tn, options, get_power_fn) == 1:
+				power_on = True
+				break
+
+		if power_on == False:
+			# this should not fail as not was fenced succesfully
 			sys.stderr.write('Timed out waiting to power ON\n')
+
 		print "Success: Rebooted"
 	elif options["-o"] == "status":
 		print "Status: " + status.upper()
@@ -768,7 +870,7 @@ def fence_login(options):
 		login_eol = "\r\n"
 
 	try:
-		re_login = re.compile("(login: )|(Login Name:  )|(username: )|(User Name :)", re.IGNORECASE)
+		re_login = re.compile("(login\s*: )|(Login Name:  )|(username: )|(User Name :)", re.IGNORECASE)
 		re_pass  = re.compile("password", re.IGNORECASE)
 
 		if options.has_key("-z"):
