@@ -22,9 +22,25 @@ BUILD_DATE="March, 2008"
 #END_VERSION_GENERATION
 
 def get_power_status(conn, options):
-	try:
+	listing = ""
+
+	try:		
 		conn.send("/S"+"\r\n")
-		conn.log_expect(options, options["-c"], int(options["-Y"]))
+
+		if isinstance(options["-c"], list):
+			re_all = list(options["-c"])
+		else:
+			re_all = [options["-c"]]
+		re_next = re.compile("Enter: ", re.IGNORECASE)
+		re_all.append(re_next)
+
+		result = conn.log_expect(options, re_all, int(options["-Y"]))
+		listing = conn.before
+		if result == (len(re_all) - 1):
+			conn.send("\r\n")
+			conn.log_expect(options, options["-c"], int(options["-Y"]))
+			listing += conn.before
+
 	except pexpect.EOF:
 		fail(EC_CONNECTION_LOST)
 	except pexpect.TIMEOUT:
@@ -32,8 +48,8 @@ def get_power_status(conn, options):
 	
 	plug_section = 0
 	outlets = {}
-	for line in conn.before.splitlines():
-		if (plug_section == 2) and line.find("|") >= 0:
+	for line in listing.splitlines():
+		if (plug_section == 2) and line.find("|") >= 0 and line.startswith("PLUG") == False:
 			plug_line = [x.strip().lower() for x in line.split("|")]
 			if len(plug_line) < len(plug_header):
 				plug_section = -1
@@ -42,7 +58,8 @@ def get_power_status(conn, options):
 				return plug_line[status_index]
 			else:
 				## We already believe that first column contains plug number
-				outlets[plug_line[0]] = (plug_line[name_index], plug_line[status_index])
+				if len(plug_line[0]) != 0:
+					outlets[plug_line[0]] = (plug_line[name_index], plug_line[status_index])
 		elif (plug_section == 1):
 			plug_section = 2
 			pass
@@ -87,7 +104,7 @@ def main():
 	## Fence agent specific defaults
 	#####
 	if 0 == options.has_key("-c"):
-		options["-c"] = [ "RSM>", "MPC>", "IPS>", "TPS>", "NBB>", "NPS>" ]
+		options["-c"] = [ "RSM>", "MPC>", "IPS>", "TPS>", "NBB>", "NPS>", "VMR>" ]
 
 	docs = { }
 	docs["shortdesc"] = "Fence agent for WTI"
@@ -96,6 +113,7 @@ which can be used with the WTI Network Power Switch (NPS). It logs \
 into an NPS via telnet or ssh and boots a specified plug. \
 Lengthy telnet connections to the NPS should be avoided while a GFS cluster \
 is running because the connection will block any necessary fencing actions."
+	docs["vendorurl"] = "http://www.wti.com"
 	show_docs(options, docs)
 	
 	##
@@ -106,7 +124,9 @@ is running because the connection will block any necessary fencing actions."
 	if 0 == options.has_key("-x"):
 		try:
 			try:
-				conn = fspawn('%s %s' % (TELNET_PATH, options["-a"]))
+				conn = fspawn(TELNET_PATH)
+				conn.send("set binary\n")
+				conn.send("open %s -%s\n"%(options["-a"], options["-u"]))
 			except pexpect.ExceptionPexpect, ex:
 				sys.stderr.write(str(ex) + "\n")
 				sys.stderr.write("Due to limitations, binary dependencies on fence agents "
